# -*- coding: utf-8 -*-
"""Value Range Widget Interface."""

# zope imports
from z3c.form.interfaces import ISelectWidget


class IValueRangeWidget(ISelectWidget):
    """Custom value range widget."""
